@@ -162,8 +162,7 @@ RS_XML(ParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
     name = CHARACTER_DATA(fileName)[0];
 #endif
     if(!isURLDoc && (name == NULL || stat(name, &tmp_stat) < 0)) {
-      PROBLEM "Can't find file %s", CHAR_DEREF(STRING_ELT(fileName, 0))
-      ERROR;
+	Rf_error("Can't find file %s", CHAR_DEREF(STRING_ELT(fileName, 0)) );
     }
   } else {
     name = strdup(CHAR_DEREF(STRING_ELT(fileName, 0)));
@@ -199,7 +198,7 @@ RS_XML(ParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
 
   if(asTextBuffer) {
       doc = useHTML ? htmlParseDoc(CHAR_TO_XMLCHAR(name), encoding) : 
-	              xmlReadMemory(name, strlen(name), NULL, encoding, parserOptions) ;
+	  xmlReadMemory(name, (int)strlen(name), NULL, encoding, parserOptions) ;
                 	  /* xmlParseMemory(name, strlen(name)) */ 
 
       if(doc != NULL) 
@@ -226,8 +225,7 @@ RS_XML(ParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
       } else
         return(stop("XMLParseError", "error in creating parser for %s", name));
 
-      PROBLEM "error in creating parser for %s", name
-      ERROR;
+      Rf_error("error in creating parser for %s", name);
   }
 
   if(TYPEOF(xinclude) == LGLSXP && LOGICAL_DATA(xinclude)[0]) {
@@ -246,8 +244,7 @@ RS_XML(ParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
    	      free((char *) name);
 
 
-	  PROBLEM "XML document is invalid"
-          ERROR;
+	  Rf_error("XML document is invalid");
       }
   }
 
@@ -372,7 +369,9 @@ RS_XML(convertXMLDoc)(const char *fileName, xmlDocPtr doc, USER_OBJECT_ converte
  
     /* Insert the name of the file being processed */
     SET_VECTOR_ELT(rdoc, FILE_ELEMENT_NAME, NEW_CHARACTER(1));
-    SET_STRING_ELT(VECTOR_ELT(rdoc, FILE_ELEMENT_NAME), 0, ENC_COPY_TO_USER_STRING(doc->name ? XMLCHAR_TO_CHAR(doc->name) : fileName));
+    SET_STRING_ELT(VECTOR_ELT(rdoc, FILE_ELEMENT_NAME), 0,
+		   ENC_COPY_TO_USER_STRING(doc->name ? (const xmlChar*)doc->name : (const xmlChar*)fileName));
+    //SET_STRING_ELT(VECTOR_ELT(rdoc, FILE_ELEMENT_NAME), 0, ENC_COPY_TO_USER_STRING(doc->name ? XMLCHAR_TO_CHAR(doc->name) : fileName));
     SET_STRING_ELT(rdoc_el_names, FILE_ELEMENT_NAME, COPY_TO_USER_STRING("file"));
 
     /* Insert the XML version information */
@@ -431,9 +430,11 @@ processNamespaceDefinitions(xmlNs *ns, xmlNodePtr node, R_XMLSettings *parserSet
   PROTECT(names = NEW_CHARACTER(n));
 
   for(n = 0, ptr = ns; ptr ; n++, ptr = ptr->next) {
-    tmp = RS_XML(createNameSpaceIdentifier)(ptr,node);
+    // protection suggested by rchk
+    tmp = PROTECT(RS_XML(createNameSpaceIdentifier)(ptr,node));
     (void) RS_XML(notifyNamespaceDefinition)(tmp, parserSettings);
     SET_VECTOR_ELT(ans, n, tmp);
+    UNPROTECT(1);
     if(ptr->prefix)
        SET_STRING_ELT(names, n, ENC_COPY_TO_USER_STRING(XMLCHAR_TO_CHAR(ptr->prefix)));
   }
@@ -483,16 +484,19 @@ getNamespaceDefs(xmlNodePtr node, int recursive)
 	  USER_OBJECT_ tmp;
 	  int i;
 
+          PROTECT(nsDef); numProtects++;
 	  while(ptr) {
-	      tmp = getNamespaceDefs(ptr, 1);
+	      PROTECT(tmp = getNamespaceDefs(ptr, 1));
 /*	  nsDef = Rf_appendList(nsDef, tmp); */
 	      if(Rf_length(tmp)) {
 		  n = Rf_length(nsDef); 
 		  PROTECT(SET_LENGTH(nsDef, n + Rf_length(tmp)));
-		  numProtects++;
 		  for(i = 0; i < Rf_length(tmp); i++) 
 		      SET_VECTOR_ELT(nsDef, n + i, VECTOR_ELT(tmp, i));
-	      }
+                  UNPROTECT(3); /* old nsDef, tmp, new nsDef */
+                  PROTECT(nsDef);
+	      } else
+   	          UNPROTECT(1); /* tmp */
 	      ptr = ptr->next;
 	  }
       }
@@ -509,8 +513,7 @@ RS_XML(internalNodeNamespaceDefinitions)(USER_OBJECT_ r_node, USER_OBJECT_ recur
   xmlNodePtr node;
 
   if(TYPEOF(r_node) != EXTPTRSXP) {
-       PROBLEM "R_internalNodeNamespaceDefinitions expects InternalXMLNode objects"
-       ERROR;
+      Rf_error("R_internalNodeNamespaceDefinitions expects InternalXMLNode objects");
     }
 
   node = (xmlNodePtr) R_ExternalPtrAddr(r_node);
@@ -879,7 +882,7 @@ addNodesToTree(xmlNodePtr node, R_XMLSettings *parserSettings)
    int ctr = 0;
    PROTECT(e = allocVector(LANGSXP, 3));
    SETCAR(e, parserSettings->converters);
-   id = NEW_CHARACTER(0);
+   PROTECT(id = NEW_CHARACTER(0));
 
    ptr = node;
 
@@ -892,7 +895,7 @@ addNodesToTree(xmlNodePtr node, R_XMLSettings *parserSettings)
       ptr = ptr->next;
    }
 
-   UNPROTECT(1);
+   UNPROTECT(2); /* e, id */
    return(ScalarInteger(ctr));
 }
 
@@ -1048,11 +1051,9 @@ notifyError(const char *msg, va_list ap, Rboolean isError)
 {
 #if 0
     if(isError) {
-	 PROBLEM "error in validating XML document"
-	 ERROR;
+	Rf_error("error in validating XML document");
     } else {
-	 PROBLEM "warning when validating XML document"
-	 ERROR;
+	Rf_error("warning when validating XML document");
     }
 
 #else
@@ -1063,11 +1064,6 @@ notifyError(const char *msg, va_list ap, Rboolean isError)
     vsnprintf(buf, BUFSIZE, msg, ap);
 
     Rf_warning(buf);
-#if 0
-    PROBLEM buf
-        WARN;
-#endif
-
 #endif
 }
 
@@ -1208,6 +1204,7 @@ RS_XML_xmlNodeChildrenReferences(USER_OBJECT_ snode, USER_OBJECT_ r_addNames, US
     xmlNodePtr ptr =  node->children;
     int addNames = LOGICAL(r_addNames)[0];
     DECL_ENCODING_FROM_NODE(node)
+    int nprot = 0;
 
     while(ptr) {
 	count++;
@@ -1216,18 +1213,20 @@ RS_XML_xmlNodeChildrenReferences(USER_OBJECT_ snode, USER_OBJECT_ r_addNames, US
 
     ptr = node->children;
 
-    PROTECT(ans = NEW_LIST(count));
-    if(addNames) 
+    PROTECT(ans = NEW_LIST(count)); nprot++;
+    if(addNames) {
 	PROTECT(names = NEW_CHARACTER(count));
+	nprot++;
+    }
 
     for(i = 0; i < count ; i++, ptr = ptr->next) {
 	SET_VECTOR_ELT(ans, i, R_createXMLNodeRef(ptr, manageMemory));
 	if(addNames)
-	    SET_STRING_ELT(names, i, ENC_COPY_TO_USER_STRING(ptr->name ? XMLCHAR_TO_CHAR(ptr->name) : ""));
+	    SET_STRING_ELT(names, i, ENC_COPY_TO_USER_STRING(ptr->name ?  ptr->name : (const xmlChar *)""));
     }
     if(addNames)
 	SET_NAMES(ans, names);
-    UNPROTECT(1 + addNames);
+    UNPROTECT(nprot);
    
     return(ans);
 }
@@ -1241,8 +1240,7 @@ R_getNodeChildByIndex(USER_OBJECT_ snode, USER_OBJECT_ r_index, USER_OBJECT_ man
     
     num = INTEGER(r_index)[0] - 1;
     if(num < 0) {
-	PROBLEM "cannot index an internal node with a negative number %d", num
-	    ERROR;
+	Rf_error("cannot index an internal node with a negative number %d", num);
     }
 	
 
@@ -1269,6 +1267,8 @@ makeSchemaReference(xmlSchemaPtr schema)
 */
 }
 
+// unused
+#define NO_XML_MEMORY_SHOW_ROUTINE 1
 
 #ifndef NO_XML_MEMORY_SHOW_ROUTINE
 void
@@ -1288,8 +1288,7 @@ RS_XML_setDocumentName(USER_OBJECT_ sdoc, USER_OBJECT_ sname)
     xmlDocPtr doc = (xmlDocPtr) R_ExternalPtrAddr(sdoc);
 
     if(!doc) {
-	PROBLEM "NULL pointer supplied for internal document"
-	    WARN;
+	Rf_warning("NULL pointer supplied for internal document");
 	return(R_NilValue);
     }
 
@@ -1311,8 +1310,7 @@ RS_XML_getDocumentName(USER_OBJECT_ sdoc)
     USER_OBJECT_ ans;
     const xmlChar *encoding;
     if(!doc) {
-	PROBLEM "NULL pointer supplied for internal document"
-	    WARN;
+	Rf_warning("NULL pointer supplied for internal document");
 	return(R_NilValue);
     }
     encoding = doc->encoding;
@@ -1348,20 +1346,19 @@ RS_XML_xmlXIncludeProcessTreeFlags(USER_OBJECT_ r_node, USER_OBJECT_ r_flags)
     xmlNodePtr node;
     int flags = INTEGER(r_flags)[0];
     int n;
-    xmlNodePtr prev, parent;
+    //xmlNodePtr prev, parent;
     SEXP ans = R_NilValue;
 
     node = (xmlNodePtr) R_ExternalPtrAddr(r_node);
-    prev = node->prev;
-    parent = node->parent;
+    //prev = node->prev;
+    //parent = node->parent;
 
     n = xmlXIncludeProcessTreeFlags(node, flags);
 
     if(n == 0)
 	return(R_NilValue);
     else if(n == -1) {
-	PROBLEM "failed in XInclude"
-        ERROR;
+	Rf_error("failed in XInclude");
     }
 
 #if 0
@@ -1475,7 +1472,7 @@ RS_XML(AttributeList)(xmlNodePtr node, R_XMLSettings *parserSettings)
    }
     SET_NAMES(ans, ans_names);
 
-    UNPROTECT(nonTrivialAttrNamespaces ? 4 : 4);
+    UNPROTECT(4);
    }
 #if 0
    else
@@ -1593,6 +1590,7 @@ addXInclude(xmlNodePtr ptr, SEXP *ans, int level, SEXP manageMemory)
 	    SEXP oans = *ans; // avoid sequence-point error
             PROTECT(*ans = SET_LENGTH(oans, len));
 	    SET_VECTOR_ELT(*ans, len - 1, R_createXMLNodeRef(ptr, manageMemory));
+	    UNPROTECT(1);
 	    return(1);
 	} else
 	    return(0);
@@ -1645,16 +1643,15 @@ R_findXIncludeStartNodes(SEXP r_root, SEXP manageMemory)
 {
     xmlNodePtr root;
     SEXP ans;
-    int count;
 
     root = (xmlNodePtr) R_ExternalPtrAddr(r_root);
     if(!root)
 	return(R_NilValue);
 
     PROTECT(ans = allocVector(VECSXP, 0));
-//    count = findXIncludeStartNodes(root, &ans, 0);
-    count = addXInclude(root, &ans, 0, manageMemory) + processKids(root, &ans, 0, manageMemory);
-    UNPROTECT(count + 1);
+    addXInclude(root, &ans, 0, manageMemory);
+    processKids(root, &ans, 0, manageMemory);
+    UNPROTECT(1);
     return(ans);
 }
 
